@@ -1,5 +1,5 @@
 /* Copyright (c) 2000  Kevin Sullivan <nite@gis.net>
-1 *
+ *
  * Please refer to the COPYRIGHT file for more information.
  */
 
@@ -32,10 +32,7 @@
  * sets_t provides a similar key-value structure: the key is again a
  * string called "nm", and the value is a pair of a string "d" and an
  * integer "t". Apparently the intention is that t==0 if d represents
- * a number, t==1 otherwise. Actually, this is implemented
- * inconsistently:
- * chset() sets t=1 iff either d contains a letter [a-zA-Z] or d=="".
- * loadsets() sets t=1 iff d contains a non-digit. 
+ * a number, t==1 otherwise.
  *
  * There is also the array vars, which is defined in vfuncs.c, and
  * maps certain names to functions.
@@ -53,14 +50,13 @@ sets_t *setl;
    can be "/set" */
 
 hardwired_t hw[] = {
-  { "user", 1, &info.user, "# your napster username" },
-  { "pass", 2, &info.pass, "# your napster password - optional" },
-  { "email", 1, &info.email, "# your napster email address" },
-  { "upload", 1, &info.up, "# list of upload directories, separated by semicolon" },
-  { "download", 1, &info.down, "# your download directory" },
-  { "incomplete", 1, &info.incomplete, "# your directory for incomplete files" },
-  { "maxuploads", 1, &info.maxuploads, "# maximal number of simultaneous uploads allowed" },
-  { "connection", 1, &info.conn,
+  { "user", 1, &info.user, 0, "# your napster username" },
+  { "pass", 2, &info.pass, 0, "# your napster password - optional" },
+  { "email", 1, &info.email, 0, "# your napster email address" },
+  { "upload", 1, &info.up, 0, "# list of upload directories, separated by semicolons" },
+  { "download", 1, &info.down, 0, "# your download directory" },
+  { "incomplete", 1, &info.incomplete, 0, "# your directory for incomplete files" },
+  { "connection", 0, 0, 0,
     "# your connection speed, according to the following chart:\n"
     "#\n"
     "# Connection | Number\n"
@@ -76,33 +72,44 @@ hardwired_t hw[] = {
     "# DSL        |  8\n"
     "# T1         |  9\n"
     "# T3 or >    | 10" },
-  { "servers", 1, &info.serverlist, "# list of servers, separated by semicolon" },
-  { "dataport", 1, &info.dataport, "# port or range of ports to use for client-client connections" },
-  { "logfile", 1, &info.logfile, "# log file for transfer logs" },
-  { "logallfile", 1, &info.logallfile, "# log file for logging everything" },
-  { "announcepongs", 0, 0, "# should receipt of PONG packets be announced?" },
-  { "autopurge", 0, 0, "# automatically delete items from up- and download lists?" },
-  { "autoreply", 0, 0, "# a string to send as an automatic reply to incoming user messages" },
-  { "connecttimeout", 0, 0, "# timeout after this many seconds while connecting to server" },
-  { "cursorfollowsscreen", 0, 0, "# should PgUp and PgDn move the screen, rather than the cursor, on the\n# search result screen?" },
-  { "debug", 0, 0, "# debug level" },
-  { "incompletesuffix", 0, 0, "# suffix to use for incomplete files" },
-  { "maxdownloads", 0, 0, "# maximum number of simultaneous downloads allowed" },
-  { "maxdownuser", 0, 0, "# maximum number of simultaneous downloads per user" },
-  { "maxupuser", 0, 0, "# maximum number of simultaneous uploads per user" },
-  { "metaserver", 0, 0, "# URL of a napigator-style metaserver" },
-  { "metatimeout", 0, 0, "# timeout for connecting to metaserver" },
-  { "newstimeout", 0, 0, "# timeout for retrieving news" },
-  { "noechosets", 0, 0, "# should the /set command be quiet?" },
-  { "nometa", 0, 0, "# skip connecting to metaserver on startup?" },
-  { "nonews", 0, 0, "# do you want to skip news about new releases on startup?" },
-  { "noresultscreen", 0, 0, "# should search results be displayed on the main screen?" },
-  { "noscroll", 0, 0, "# should main screen not scroll to bottom automatically on output?" },
-  { "savechannels", 0, 0, "# should your open channels to be saved between sessions?" },
-  { "savepass", 0, 0, "# should password be saved in config file?" },
-  { "sdefaults", 0, 0, "# default keystrokes for search result screen appearance" },
-  { "showtoomanyuploads", 0, 0, "# display a message when upload limit is reached?" },
-  { "turdsize", 0, 0, "# size of the largest incomplete file which will be auto-removed" }
+  { "maxuploads", 0, 0, 0, "# maximal number of simultaneous uploads allowed" },
+  { "maxupuser", 0, 0, 0, "# maximum number of simultaneous uploads per user" },
+  { "maxdownloads", 0, 0, 0, "# maximum number of simultaneous downloads allowed" },
+  { "maxdownuser", 0, 0, 0, "# maximum number of simultaneous downloads per user" },
+  { "servers", 1, &info.serverlist, 0,
+    "# list of servers, separated by semicolons (note: this is now ignored\n"
+    "# and overwritten unless nometa is set)" },
+  { "dataport", 1, &info.dataport, 0, "# port or range of ports to use for client-client connections.\n# Set this to 0 if you are behind a firewall" },
+  { "logfile", 1, &info.logfile, 0, "# log file for transfer logs" },
+  { "logallfile", 1, &info.logallfile, 0, "# log file for logging everything" },
+  { "bandwidthdown", 0, 0, &info.bandwidthdown, "# global bandwidth limit for downloads (in kB/s)" },
+  { "bandwidthdown1", 0, 0, &info.bandwidthdownconn, "# download bandwidth limit per connection (in kB/s)" },
+  { "bandwidthup", 0, 0, &info.bandwidthup, "# global bandwidth limit for uploads (in kB/s)" },
+  { "bandwidthup1", 0, 0, &info.bandwidthupconn, "# upload bandwidth limit per connection (in kB/s)" },
+
+  { "announcepongs", 0, 0, 0, "# should receipt of PONG packets be announced?" },
+  { "autopurge", 0, 0, 0, "# automatically delete items from up- and download lists?" },
+  { "autoreply", 0, 0, 0, "# a string to send as an automatic reply to incoming user messages" },
+  { "connecttimeout", 0, 0, 0, "# timeout after this many seconds while connecting to server" },
+  { "cursorfollowsscreen", 0, 0, 0, "# should PgUp and PgDn move the screen, rather than the cursor, on the\n# search result screen?" },
+  { "debug", 0, 0, 0, "# debug level" },
+  { "incompletesuffix", 0, 0, 0, "# suffix to use for incomplete files" },
+  { "libraryfile", 1, &info.shared_filename, 0, "# location of your library file" },
+  { "metaserver", 0, 0, 0, "# URL of a napigator-style metaserver" },
+  { "metatimeout", 0, 0, 0, "# timeout for connecting to metaserver" },
+  { "napping", 0, 0, 0, "# relative or absolute filename of the napping program to use\n" },
+  { "newstimeout", 0, 0, 0, "# timeout for retrieving news" },
+  { "noechosets", 0, 0, 0, "# should the /set command be quiet?" },
+  { "nometa", 0, 0, 0, "# skip connecting to metaserver on startup?" },
+  { "nonews", 0, 0, 0, "# do you want to skip news about new releases on startup?" },
+  { "noping", 0, 0, 0, "# do you want to disable pings?" },
+  { "noresultscreen", 0, 0, 0, "# should search results be displayed on the main screen?" },
+  { "noscroll", 0, 0, 0, "# should main screen not scroll to bottom automatically on output?" },
+  { "savechannels", 0, 0, 0, "# should your open channels to be saved between sessions?" },
+  { "savepass", 0, 0, 0, "# should password be saved in config file?" },
+  { "sdefaults", 0, 0, 0, "# default keystrokes for search result screen appearance" },
+  { "showtoomanyuploads", 0, 0, 0, "# display a message when upload limit is reached?" },
+  { "turdsize", 0, 0, 0, "# size of the largest incomplete file which will be auto-removed" }
 };
 
 int hwsize = sizeof(hw)/sizeof(hardwired_t);
@@ -274,30 +281,42 @@ void addset(const char *nm, const char *val)
 
   /* check if this is a hardwired set */
   for (i=0; i<hwsize; i++) {
-    if (hw[i].type && !strcasecmp(nm, hw[i].nm)) {
-      free(*hw[i].strp);
-      *hw[i].strp = strdup(val);
-      return;
+    if (!strcasecmp(nm, hw[i].nm)) 
+      break;
+  }
+
+  if (i<hwsize && hw[i].type) {
+    free(*hw[i].strp);
+    *hw[i].strp = strdup(val);
+    if (hw[i].intp) {
+      *hw[i].intp = strtol(val, &endptr, 10);
+      if (!endptr || *endptr!='\0') { /* string was invalid number */
+	*hw[i].intp = 0;
+      }
     }
+    return;
   }
 
   /* make new assignment */
   elt = (sets_t *)malloc(sizeof(sets_t));
   elt->nm = strdup(nm);
   elt->d = strdup(val);
-  elt->intval = strtol(elt->d, &endptr, 10);
-  if (endptr && *endptr=='\0') { /* string was valid number */
-    elt->t = 0;
-  } else {
+  elt->intval = strtol(val, &endptr, 10);
+  elt->t = 0;
+  if (!endptr || *endptr!='\0') { /* string was invalid number */
     elt->intval = 0;
     elt->t = 1;
   }
 
+  if (i<hwsize && hw[i].intp) {
+    *hw[i].intp = elt->intval;
+  }
+
   /* now append it to the list */
   list_append(sets_t, setl, elt);
 }
 
-/* add a set to the list, but remove duplicate first. Set t=1 if this
+/* add a set to the list, but remove duplicate first. Set t=0 if this
    might be a number. If d is NULL, just delete the set. */
 void chset(const char *nm, const char *d)
 {
@@ -364,11 +383,17 @@ void delset(const char *nm)
 
   /* check for hardwired sets first */
   for (i=0; i<hwsize; i++) {
-    if (!strcasecmp(nm, hw[i].nm) && hw[i].type != 0) {
-      free(*hw[i].strp);
-      *hw[i].strp = NULL;
-      return;
+    if (!strcasecmp(nm, hw[i].nm)) 
+      break;
+  }
+
+  if (i<hwsize && hw[i].type != 0) {
+    free(*hw[i].strp);
+    *hw[i].strp = NULL;
+    if (hw[i].intp) {
+      *hw[i].intp = 0;
     }
+    return;
   }
 
   for (cur=setl;cur&&strcasecmp(cur->nm, nm);cur=cur->next)
@@ -387,10 +412,13 @@ void delset(const char *nm)
   free(cur->d);
   free(cur->nm);
   free(cur);
+  
+  if (i<hwsize && hw[i].intp) {
+    *hw[i].intp = 0;
+  }
 }
 
-/* lookup set's value as a number, or -1 if we decided it isn't 
-   a number, or if it isn't in the list */
+/* lookup set's value as a number, or -1 if not set or a non-number. */
 long nvar(const char *nm)
 {
   sets_t *cur;
@@ -408,7 +436,7 @@ long nvar(const char *nm)
     return(-1);
   
   if (cur->t)
-    return(0);
+    return(-1);
   
   return(cur->intval);
 }
