@@ -60,25 +60,6 @@ struct scount_s
 };
 typedef struct scount_s scount_t;
 
-#define ICMP_ECHO 8
-struct icmphdr 
-{
-  unsigned char type, code;
-  unsigned short checksum;
-  union
-  {
-    struct
-    {
-      unsigned short id, sequence;
-    } echo;
-    unsigned long gateway;
-    struct
-    {
-      unsigned short blah, mtu;
-    } frag;
-  } un;
-};
-
 /* ---------------------------------------------------------------------- */
 /* the data structure for search result items. Note: the convention is
    that the items of the search result list are numbered sequentially,
@@ -98,8 +79,8 @@ struct ssearch_s
 			       byte order (formerly this field was
 			       called ip and was in non-host byte
 			       order) */
-  struct timeval s, r;      /* time ping sent, received */
-  int ping;                 /* ping time */
+  int ping;                 /* ping time (in milliseconds). -3 or -2 if none
+                               requested, -1 if infinity */
   struct ssearch_s *next;   /* next item in list */
   struct ssearch_s *next1;  /* only used for sorting; need not initialize */
 };
@@ -422,6 +403,41 @@ struct download_s
 };
 typedef struct download_s download_t;
 
+/* ---------------------------------------------------------------------- */
+/* struct to hold state for an incoming direct browse connection (i.e. one 
+ * requested by us). This is similar to a download task, but because we only 
+ * allow one direct browse request at a time, we use a single global variable
+ * called directbrowse (initialized in cmds.c) to store the state for the
+ * current request. (In comparison, download task structures are kept in a
+ * linked list.) */ 
+struct inbrowse_s {
+  int state;      /* States: REQUESTED, CONNECTING, IN_PROGRESS, TIMED_OUT,
+                     COMPLETE, FAILED. */
+  struct sock_s *sk;
+  time_t reqtime; /* time direct browse request was sent */
+  char *nick;     /* nick of remote client we want to browse */
+  FILE *f;        /* stream associated with socket's file descriptor */
+};
+typedef struct inbrowse_s inbrowse_t;
+
+/* ---------------------------------------------------------------------- */
+/* struct to hold state for an outgoing direct browse connection (i.e. 
+ * requested by a remote client). The sock_t for the connection keeps a
+ * pointer to this structure. This is similar to an upload task. However, 
+ * we don't create the structure upon accepting a direct browse request,
+ * because there is no way to associate an incoming GETLIST command with the
+ * nick of the requester. (For uploads, the remote client identifies itself
+ * in the GET command, so an upload task in the WAITING state can be linked to
+ * the connection's sock_t structure.) 
+ * The outbrowse_t structure is created when the connection is already in the
+ * IN_PROGRESS state, so we don't bother storing the actual state name at all */
+struct outbrowse_s {
+  FILE *g; /* stream associated with the shared library file 
+              (each connection will be at a different position in the file) */
+  /* may need more fields later... */
+};
+typedef struct outbrowse_s outbrowse_t;
+
 #define O_NAP_FUNC(x) int (x)(int s, char *str, char **tok, int num, WINDOW *win)
 
 O_NAP_FUNC(ddebug);
@@ -461,6 +477,7 @@ O_NAP_FUNC(dunmuzzle);
 O_NAP_FUNC(ddisconnect);
 O_NAP_FUNC(dreconnect);
 O_NAP_FUNC(dbrowse);
+O_NAP_FUNC(dbrowse2);
 O_NAP_FUNC(dhelp);
 O_NAP_FUNC(dping);
 O_NAP_FUNC(duserpass);
@@ -573,6 +590,7 @@ void rqueued_hook(char *nick);
 int save_hotlist(WINDOW *win);
 int metaserver(char *url, int timeout, const char **errmsg);
 FILE *open_url(const char *url, int timeout, const char **errmsg);
+char *format_time(time_t t);
 
 #endif /* _NAP_CMDS_H */
 
