@@ -37,6 +37,8 @@
 extern upload_t *up;
 extern download_t *down;
 extern info_t info; 
+extern int srch; /* 1 while search or browse is in progress */
+extern struct inbrowse_s directbrowse; 
 
 int noprint=0;
 
@@ -136,11 +138,13 @@ int inserv(WINDOW *win, sock_t *m)
   else if (r == -1) {
     wp(win, ""RED"* Error: %s"WHITE"\n", strerror(errno));
     drw(win);
-    return(0);
+    delsock(m->fd);
+    return(1);
   } else if (r == -3) {
     wp(win, ""RED"* Error: unexpected end of file from server"WHITE"\n");
     drw(win);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
 
   for (j=0;data[j];j++)
@@ -152,10 +156,14 @@ int inserv(WINDOW *win, sock_t *m)
     
   n = parsein(s, pk->op, data, win);
   noprint = 0;
-  if (n == -1)
-    return(0);
+  if (n == -1) {
+    delsock(m->fd);
+    return(1);
+  }
   else if (n == -2)
-    return(-1);
+    return(-1);      /* don't know what this does, or used to
+			do. Don't know of a concrete situation where
+			parsein returns -2, or in fact -1. */
   
   free(pk);
   free(data);
@@ -183,21 +191,47 @@ int gconn(WINDOW *win, sock_t *m)
   return(1);
 }
 
-/* a remote client has connected to us and is about to say "GET" or "SEND" */
+/* a remote client has connected to us and is about to say "GET", "SEND",
+ * "GETLIST", or "SENDLIST" (the latter two are for direct browsing) */
+
 int lsock(WINDOW *win, sock_t *m)
 {
-  char buf[5];
+  char buf[8];
   int r;
 
   memset(buf, 0, sizeof(buf));
-  r = recv(m->fd, &buf, 4, MSG_PEEK);
+  r = recv(m->fd, &buf, sizeof(buf), MSG_PEEK);
 
   if (r==-1)
   {
     wp(win, ""RED"* Bogus connection from remote client (%s): %s"WHITE"\n", getpeerip(m->fd), strerror(errno));
     drw(win);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
+
+  /* GET and SEND are prefixes of GETLIST and SENDLIST respectively,
+   * so have to check for the longer words first */
+  if (!strncmp(buf, "GETLIST", 7))
+  {
+    recv(m->fd, &buf, 7, 0);
+    m->func = dogetlist;
+    m->t = S_W; /* prepare to write */
+  }
+  else if (!strncmp(buf, "SENDLIST", 8))
+  {
+    recv(m->fd, &buf, 8, 0);
+    /* we rename the connection here so that we can find it and delete it
+     * if the direct browse request times out. This is necessary because
+     * a broken client like Napster v2.0 BETA 9.6 will send a SENDLIST and
+     * nothing else. As a result, dosendlist will never be called, and the 
+     * socket will remain open until the program quits. */
+    free(m->nm);
+    m->nm = strdup("sendlist");
+    m->func = dosendlist;
+  }
+  else              /* note: trailing "else" goes with following "if"!! */
+
   if (!strncmp(buf, "GET", 3))
   {
     recv(m->fd, &buf, 3, 0);
@@ -221,7 +255,8 @@ int lsock(WINDOW *win, sock_t *m)
     ssock(m->fd, "INVALID REQUEST\n");
 
     free(data);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
     
   return(1);
@@ -240,7 +275,8 @@ int doget(WINDOW *win, sock_t *m)
   FILE *f;
   
   if (rsock(m->fd, &buf) == -1) {
-    return(0);
+    delsock(m->fd);
+    return(1);
   }    
 
   tok = form_tokso(buf, &cnt); 
@@ -259,7 +295,8 @@ int doget(WINDOW *win, sock_t *m)
     for (i=0;i<cnt;i++)
       free(tok[i]);
     free(tok);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
   
   nick = strdup(tok[0]);
@@ -282,10 +319,10 @@ int doget(WINDOW *win, sock_t *m)
     /* wp(win, "%s* %s requested upload of \"%s\" but it isn't in the upload list! (GET)%s\n", RED, nick, rfn, WHITE);
        drw(win); */
     ssock(m->fd, "INVALID REQUEST\n");
-    /* close(m->fd); */
     free(nick);
     free(rfn);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
   
   free(nick);
@@ -306,7 +343,8 @@ int doget(WINDOW *win, sock_t *m)
     ssock(m->fd, "FILE NOT FOUND\n");
     task->state = FAILED;
     task->d_time = time(0);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
   
   /* determine file size */
@@ -320,23 +358,26 @@ int doget(WINDOW *win, sock_t *m)
     fclose(f);
     task->state = FAILED;
     task->d_time = time(0);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
   
   fseek(f, bsz, SEEK_SET); 
   
   /* go to state IN_PROGRESS */
   
-  task->state = IN_PROGRESS;
   task->sk = m;
   task->f = f;
   task->size = st.st_size;
   task->bsz = bsz;
   task->pos = bsz;
   task->p_time = time(0);
+  task->state = IN_PROGRESS;
 
   /* link task to socket */
   m->utask = task;
+  m->bwlimit = 1;
+  bandwidth_init(&m->bw);
   m->func = sfile;
   m->t = S_W;
   free(m->nm);
@@ -369,8 +410,8 @@ int dosend(WINDOW *win, sock_t *m)
   
   if (rsock(m->fd, &buf) == -1)
   {
-    /* close(m->fd); */
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
 
   tok = form_tokso(buf, &cnt);
@@ -384,11 +425,11 @@ int dosend(WINDOW *win, sock_t *m)
     drw(win);
     ssock(m->fd, "INVALID REQUEST\n");
     free(buf);
-    /* close(m->fd); */
     for (i=0;i<cnt;i++)
       free(tok[i]);
     free(tok);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
   
   nm = strdup(tok[0]);
@@ -413,8 +454,8 @@ int dosend(WINDOW *win, sock_t *m)
        (and going by IP address would be unreliable) */
     free(nm);
     free(rfn);
-    /* close(m->fd); */
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
   
   list_find(task, down, !strcasecmp(task->nick, nm) && !strcasecmp(task->rfn, rfn) && task->state==WAITING);
@@ -436,8 +477,8 @@ int dosend(WINDOW *win, sock_t *m)
     }
     free(rfn);
     free(nm);
-    /* close(m->fd); */
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
   
   if (!task)
@@ -447,10 +488,10 @@ int dosend(WINDOW *win, sock_t *m)
        error message, since it is confusing and not useful */
     /*    wp(win, "%s* Error: %s tried to send (push) \"%s\", but we did not request it!%s\n", RED, nm, fn, WHITE);
 	  drw(win); */
-    /* close(m->fd); */
     free(nm);
     free(rfn);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
 
   /* try to open local file for this download */
@@ -471,7 +512,8 @@ int dosend(WINDOW *win, sock_t *m)
     free(task->check);
     task->state = FAILED;
     task->d_time = time(0);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
 
   free(nm);
@@ -490,6 +532,8 @@ int dosend(WINDOW *win, sock_t *m)
   free(m->nm);
   m->nm = strdup(gnum(0));
   m->dtask = task;
+  m->bwlimit = 1;
+  bandwidth_init(&m->bw);
   m->func = gfile;
   sv = findsock("server");
   if (sv)
@@ -582,7 +626,8 @@ int initsend(WINDOW *win, sock_t *m)
     drw(win);
     task->state=FAILED;
     task->d_time = time(0);
-    return(0);      /* let sockfunc() delete socket for this task */
+    delsock(m->fd);
+    return(1);
   }
 
   /* Open the local file for reading. Note: we already checked that
@@ -596,7 +641,8 @@ int initsend(WINDOW *win, sock_t *m)
     drw(win);
     task->state=FAILED;
     task->d_time = time(0);
-    return(0);      /* let sockfunc() delete socket for this task */
+    delsock(m->fd);
+    return(1);
   }
   
   /* determine file size */
@@ -617,7 +663,8 @@ int initsend(WINDOW *win, sock_t *m)
     fclose(f);
     task->state=FAILED;
     task->d_time = time(0);
-    return(0);      /* let sockfunc() delete socket for this task */
+    delsock(m->fd);
+    return(1);
   }
   
   /* go to state CONNECTING1, continue with ssize() */
@@ -650,7 +697,8 @@ int ssize(WINDOW *win, sock_t *m)
     fclose(task->f);
     task->state = FAILED;
     task->d_time = time(0);
-    return(0);   /* let sockfunc() close this socket */
+    delsock(m->fd);
+    return(1);
   }
 
   bsz = strtoul(buf, 0, 10);
@@ -661,7 +709,8 @@ int ssize(WINDOW *win, sock_t *m)
     fclose(task->f);
     task->state = FAILED;
     task->d_time = time(0);
-    return(0);   /* let sockfunc() close this socket */
+    delsock(m->fd);
+    return(1);
   }
   
   fseek(task->f, bsz, SEEK_SET);
@@ -673,6 +722,8 @@ int ssize(WINDOW *win, sock_t *m)
   task->p_time = time(0);
   task->state = IN_PROGRESS;
   
+  m->bwlimit = 1;
+  bandwidth_init(&m->bw);
   m->func = sfile;
   m->t = S_W;
   
@@ -701,16 +752,18 @@ int initget(WINDOW *win, sock_t *m)
     free(task->check);
     task->state = FAILED;
     task->d_time = time(0);
-    /* close(m->fd); */
-    return(0);
+    delsock(m->fd);
+    return(1);
   } 
   
   /* make sure the task is not stale */
   list_find(elt, down, elt==task);
   
-  if (!elt)
-    return(0);
-  
+  if (!elt) {
+    delsock(m->fd);
+    return(1);
+  }
+
   r = ssock(s, "GET");
   r = ssock(s, "%s \"%s\" %lu", info.user, task->rfn, 0);
   
@@ -724,8 +777,8 @@ int initget(WINDOW *win, sock_t *m)
     free(task->check);
     task->state = FAILED;
     task->d_time = time(0);
-    /* close(m->fd); */
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
   
   m->func = gsize;
@@ -761,8 +814,8 @@ int gsize(WINDOW *win, sock_t *m)
       free(task->check);
       task->state = FAILED;
       task->d_time = time(0);
-      /* close(m->fd); */
-      return(0);
+      delsock(m->fd);
+      return(1);
     }
     /* abort only if first character is not a digit.
      * this catches messages such as "INVALID REQUEST" or "FILE NOT SHARED"
@@ -779,8 +832,8 @@ int gsize(WINDOW *win, sock_t *m)
       free(task->check);
       task->state = FAILED;
       task->d_time = time(0);
-      /* close(m->fd); */
-      return(0);
+      delsock(m->fd);
+      return(1);
     }
     if (!isdigit(buf[i]))
       break;
@@ -813,7 +866,8 @@ int gsize(WINDOW *win, sock_t *m)
     free(task->check);
     task->state = FAILED;
     task->d_time = time(0);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
 
   task->lfn = lfn;
@@ -824,6 +878,8 @@ int gsize(WINDOW *win, sock_t *m)
   task->p_time = time(0);
   task->state = IN_PROGRESS;
 
+  m->bwlimit = 1;
+  bandwidth_init(&m->bw);
   m->func = gfile;
 
   return(1);
@@ -866,8 +922,9 @@ int sfile(WINDOW *win, sock_t *m)
     fclose(task->f);
     task->d_time = t;
     task->state = COMPLETE;
-    
-    return(0);  /* let sockfunc() close this socket */
+
+    delsock(m->fd);
+    return(1);
   }
   
   r = send(m->fd, buf, n, 0);
@@ -901,15 +958,378 @@ int sfile(WINDOW *win, sock_t *m)
     fclose(task->f);
     task->d_time = t;
     task->state = INCOMPLETE;
-    
-    return(0);  /* let sockfunc() close this socket */
+
+    delsock(m->fd);
+    return(1);
   }
   
   task->pos += n;
+
+  /* update byte counts for bandwidth limiting */
+  bandwidth_register(&m->bw, n);
+  bandwidth_register(&bwup, n);
   
   return(1);
 }
 
+/* ------------------------------------------------------------------------ */
+/* we have connected to a remote client in order to issue a GETLIST command.
+ * The connection was initiated in scmds.c:sbrowse2acc(). */
+int initgetlist(WINDOW *win, sock_t *m)
+{
+  int s = m->fd, r;
+  char c;
+ 
+  /* remote client should send a '1' (ASCII character 49) */
+  r = recv(s, &c, 1, 0);
+  if (r <= 0) {
+    wp(win, ""RED"* Error browsing %s: %s"WHITE"\n", directbrowse.nick, \
+        r ? strerror(errno) : "connection refused");
+    drw(win);
+    directbrowse.state = FAILED;
+    srch = 0;
+    delsock(m->fd);
+    return(1);
+  } 
+  
+  r = ssock(s, "GETLIST");
+  if (r <= 0) {
+    wp(win, ""RED"* Error browsing %s: %s"WHITE"\n", directbrowse.nick, \
+        r ? strerror(errno) : "0 bytes written to socket");
+    drw(win);
+    directbrowse.state = FAILED;
+    srch = 0;
+    delsock(m->fd);
+    return(1);
+  }
+ 
+  /* still in CONNECTING state */
+  m->func = glistnick; /* wait for remote client to send its nick */
+  return(1);
+}
+
+/* ------------------------------------------------------------------------ */
+/* after we issue a GETLIST command, the remote client sends it nick before
+ * sending its shared file list. */
+int glistnick(WINDOW *win, sock_t *m)
+{
+  char *nick = NULL;
+  FILE *f;
+  
+  /* we duplicate the file descriptor so the new descriptor can be closed 
+   * independently of the original one when fclose() is called. */
+  directbrowse.f = f = fdopen(dup(m->fd), "r");
+  /* remote client sends its nick as "<nick>\n" */
+  nick = getline(f); /* getline uses fgets() to read a line */
+  if (!nick) {
+    wp(win, ""RED"* Error browsing %s: remote client did not send its nick"\
+        WHITE"\n", directbrowse.nick);
+    drw(win);
+    directbrowse.state = FAILED;
+    srch = 0;
+    fclose(f);
+    /* closing the stream will also close the associated file descriptor,
+     * but remember that this descriptor is a duplicate of m->fd, so 
+     * we can safely close m->fd. */
+    delsock(m->fd);
+    return(1);
+  }
+  /* is this who we were expecting? */
+  if (strcasecmp(directbrowse.nick, nick) != 0) {
+    wp(win, ""RED"* Error browsing %s: remote client reports nick as %s"\
+        WHITE"\n", directbrowse.nick, quote(nick));
+    drw(win);
+    free(nick);
+    directbrowse.state = FAILED;
+    srch = 0;
+    fclose(f);
+    delsock(m->fd);
+    return(1);
+  }
+
+  free(nick);
+  
+  wp(win, "* Receiving shared file list from %s...\n", directbrowse.nick);
+  drw(win);
+  directbrowse.state = IN_PROGRESS;
+  m->func = glist;
+  return(1);
+}
+
+/* ------------------------------------------------------------------------ */
+/* receive a shared file list from a remote client, one line per call */
+int glist(WINDOW *win, sock_t *m)
+{
+  char *buf;
+  char **tok;
+  char *tmptok[7];
+  int cnt, i;
+  FILE *f = directbrowse.f;
+
+  /* remote client sends a list of files consisting of lines like
+   * "<filename>" <md5> <size> <bitrate> <frequency> <time>\n 
+   * Last line is a \n by itself. */
+
+  buf = getline(f); /* getline will strip the trailing newline */
+  /* last line (single newline) will be returned as an empty string */
+  if (buf && *buf != '\0') {
+    if (nvar("debug") == 2) {
+      /* trailing newline was already stripped, so it won't show up here */
+      wp(win, ""DARK GREEN"<-- [from %d=%s] <%s>"WHITE"\n", m->fd, \
+          m->nm, quote(buf));
+      drw(win);
+    }
+    tok = form_tokso(buf, &cnt);
+    /* expect exactly 6 tokens */
+    if (cnt == 6) {
+        /* srbrowse expects 7 tokens */
+        tmptok[0] = directbrowse.nick;
+        for (i = 0; i < 6; i++) {
+          tmptok[i+1] = tok[i];
+        }
+        /* re-use the function that processes browse responses from the
+         * server. The first arg is normally the file descriptor for the
+         * socket connected to the server, but it isn't used by srbrowse
+	 * anyway. */
+        srbrowse(m->fd, buf, tmptok, 7, win);
+    }
+    for (i = 0; i < cnt; i++)
+      free(tok[i]);
+    free(tok);
+    free(buf);
+  } else {
+    free(buf);
+    fclose(f);
+    directbrowse.state = COMPLETE;
+    showresults(win, 0); /* also resets srch to 0 */
+    delsock(m->fd);
+    return(1);
+  }
+  
+  return(1);
+} 
+
+/* ------------------------------------------------------------------------ */
+/* a remote client connected to us and issued a SENDLIST command so that it 
+ * can send us its shared file list. */
+int dosendlist(WINDOW *win, sock_t *m)
+{
+  char *nick = NULL;
+  FILE *f;
+  
+  /* were we expecting a browse list? */
+  if (directbrowse.state != REQUESTED) {
+    delsock(m->fd);
+    return(1);
+  }
+  
+  /* "<nick>\n" immediately follows the SENDLIST command. */
+  directbrowse.f = f = fdopen(dup(m->fd), "r");
+  nick = getline(f); /* getline will strip leading spaces if there are any */
+
+  /* is this who we were expecting? */
+  if (!nick || strcasecmp(directbrowse.nick, nick) != 0) {
+    if (nvar("debug") == 2) {
+      wp(win, ""DARK GREEN"<-- [from %d=%s] Got unexpected SENDLIST from %s"\
+          " (%s)"WHITE"\n", m->fd, m->nm, nick ? quote(nick):"?", \
+          getpeerip(m->fd));
+      drw(win);
+    }
+    free(nick);
+    /* directbrowse.state remains REQUESTED */
+    fclose(f);
+    delsock(m->fd);
+    return(1);
+  }
+  
+  free(nick);
+
+  /* rename the connection */
+  free(m->nm);
+  m->nm = strdup("dirbrowse");
+  
+  if (nvar("debug") == 2) {
+    wp(win, ""DARK GREEN"<-- [from %d=%s] Got SENDLIST from %s"WHITE"\n", \
+        m->fd, m->nm, directbrowse.nick);
+  }
+  wp(win, "* Receiving shared file list from %s...\n", directbrowse.nick);
+  drw(win);
+
+  directbrowse.sk = m;
+  directbrowse.state = IN_PROGRESS;
+  m->func = glist;
+  return(1);
+}
+
+/* ------------------------------------------------------------------------ */
+/* a remote client connected to us and issued a GETLIST command
+ * (this is an outgoing direct browse connection) */
+int dogetlist(WINDOW *win, sock_t *m)
+{
+  int s = m->fd, r;
+  FILE *g;
+  char *libraryfile;
+
+  /* XXX if we limit the number of outgoing browse connections,
+   * check that we are within the limit here. */
+
+  if (nvar("debug") == 2) {
+    wp(win, ""DARK GREEN"<-- [from %d=%s] Got GETLIST"WHITE"\n", m->fd, m->nm);
+    drw(win);
+  }
+
+  /* rename the connection to "b #", where # is an integer */
+  free(m->nm);
+  m->nm = strdup(gnum(2));
+
+  /* send "<mynick>\n" */
+  r = ssock(s, "%s\n", info.user);
+  if (r <= 0) {                         
+    if (nvar("debug") == 2) {
+      wp(win, ""RED"* Error sending list: %s"WHITE"\n", \
+          r ? strerror(errno) : "0 bytes written to socket");
+      drw(win);
+    }
+    delsock(m->fd);
+    return(1);
+  }       
+  
+  if (info.shared_filename) {
+    libraryfile = info.shared_filename;
+  } else {
+    libraryfile = home_file(LIBRARYFILE);
+  }
+
+  g = fopen(libraryfile, "r");
+  if (!g) {
+    if (nvar("debug") == 2) {
+      wp(win, ""RED"* Error sending list: cannot open %s (%s)"WHITE"\n", \
+          libraryfile, strerror(errno));
+      drw(win);
+    }
+    delsock(m->fd);
+    return(1);
+  }
+  
+  m->btask = (outbrowse_t *) malloc(sizeof(outbrowse_t));
+  m->btask->g = g;
+  
+  m->func = slist;
+  m->t = S_W;
+  return(1);
+}
+
+/* ------------------------------------------------------------------------ */
+/* we have connected to a remote client to issue a SENDLIST command and 
+ * send our shared file list. (this is an outgoing direct browse connection)
+ * The connection was initiated in scmds.c:sbrowse2req() and is named "b #",
+ * where # is an integer. */
+int initsendlist(WINDOW *win, sock_t *m)
+{
+  int s = m->fd, r;
+  char c;
+  FILE *g;
+  char *libraryfile;
+ 
+  /* remote client should send a '1' */
+  r = recv(s, &c, 1, 0);
+  if (r <= 0) {
+    if (nvar("debug") == 2) {
+      wp(win, ""RED"* Error sending list: %s"WHITE"\n", \
+        r ? strerror(errno) : "connection refused");
+      drw(win);
+    }
+    delsock(m->fd);
+    return(1);
+  } 
+  
+  /* send "SENDLIST" then "<mynick>\n" (this is what the Napster v2.0 BETA 8
+   * and BETA 9 clients seem to do; there is no space in between.) */
+  r = ssock(s, "SENDLIST");
+  r = ssock(s, "%s\n", info.user);
+  if (r <= 0) {
+    if (nvar("debug") == 2) {
+      wp(win, ""RED"* Error sending list: %s"WHITE"\n", \
+          r ? strerror(errno) : "0 bytes written to socket");
+      drw(win);
+    }
+    delsock(m->fd);
+    return(1);
+  }
+  
+  if (info.shared_filename) {
+    libraryfile = info.shared_filename;
+  } else {
+    libraryfile = home_file(LIBRARYFILE);
+  }
+
+  g = fopen(libraryfile, "r");
+  if (!g) {
+    if (nvar("debug") == 2) {
+      wp(win, ""RED"* Error sending list: cannot open %s (%s)"WHITE"\n", \
+          libraryfile, strerror(errno));
+      drw(win);
+    }
+    delsock(m->fd);
+    return(1);
+  }
+
+  m->btask = (outbrowse_t *) malloc(sizeof(outbrowse_t));
+  m->btask->g = g;
+ 
+  m->func = slist;
+  m->t = S_W;
+  return(1);
+}
+
+/* ------------------------------------------------------------------------ */
+/* send shared file list, one line per call */
+int slist(WINDOW *win, sock_t *m)
+{
+  int s = m->fd, r;
+  FILE *g;
+  char *buf, *p;
+  
+  if (!m->btask) {
+    delsock(m->fd);
+    return(1); /* fatal error */
+  }
+  g = m->btask->g;
+  buf = getline(g); /* getline strips the trailing newline */
+  if (!buf) { /* EOF */
+    r = ssock(s, "\n"); /* terminate the list, ignore errors */
+    fclose(g);
+    free(m->btask);
+    delsock(m->fd);
+    return(1);
+  }
+  if (*buf != '\"') { 
+    free(buf);
+    return(1); /* skip headers and junk */
+  }
+  for (p=buf; *p; p++) {
+    if (*p == '/')
+      *p = '\\';
+  }
+  /* send the line; don't forget to terminate with a newline */
+  r = ssock(s, "%s\n", buf);
+  if (r <= 0) {
+    if (nvar("debug") == 2) {
+      wp(win, ""RED"* Error sending list: %s"WHITE"\n",  \
+          r ? strerror(errno) : "0 bytes written to socket");
+      drw(win);
+    }
+    free(buf);
+    fclose(g);
+    free(m->btask);
+    delsock(m->fd);
+    return(1);
+  }
+ 
+  free(buf);
+  return(1);
+}
+
 /* receive a raw file on socket, after initget and gsize did the
    initial communication. */
 int gfile(WINDOW *win, sock_t *m)
@@ -956,11 +1376,17 @@ int gfile(WINDOW *win, sock_t *m)
       }
     }
     
-    return(0);  
+    delsock(m->fd);
+    return(1);
   }
   write(fileno(task->f), buf, n);
   
   task->pos += n;
+
+  /* update byte counts for bandwidth limiting */
+  bandwidth_register(&m->bw, n);
+  bandwidth_register(&bwdown, n);
   
   return(1);
 }
+
