@@ -28,6 +28,7 @@ extern chans_t *chanl, *curchan, *recent;
 extern int wmode;
 extern int tind;
 extern int screen;
+extern int lpbrk;
 
 int ircsock = 0;
 char *nbuf;
@@ -119,8 +120,8 @@ int dccfunc(WINDOW *win, sock_t *m)
     {
       wp(win, "%s* DCC Connection to %s closed%s\n", RED, nm, WHITE);
       drw(win);
-      close(m->fd);
-      return(0);
+      delsock(m->fd);
+      return(1);
     }
     if (msg[i] == '\n')
     {
@@ -147,8 +148,8 @@ int ldcc(WINDOW *win, sock_t *m)
   s = accept(m->fd, (struct sockaddr *)&frm, &frmlen);
   if (s == -1)
   {
-    close(m->fd);
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
   
   msprintf(&nm, "chat %s", strchr(m->nm, ' ')+1);
@@ -159,8 +160,8 @@ int ldcc(WINDOW *win, sock_t *m)
   wp(win, "%s* DCC Connection to %s established%s\n", BRIGHT(CYAN), strchr(m->nm, ' ')+1, WHITE);
   drw(win);
   
-  close(m->fd);
-  return(0);
+  delsock(m->fd);
+  return(1);
 }
 
 int inirc(WINDOW *win, sock_t *m)
@@ -191,7 +192,8 @@ int inirc(WINDOW *win, sock_t *m)
 
     ircmode = 0;
     mnick = NULL;
-    return(0);
+    delsock(m->fd);
+    return(1);
   }
   
   if (!strchr(buf, '\n') && !(*ircbuf))
@@ -265,11 +267,17 @@ int inirc(WINDOW *win, sock_t *m)
 	indraw();
       }
       dstatus();
-      return(r);
+      if (!r) {
+	delsock(m->fd);
+      }
+      return(1);
     }
   }
   
-  return(r);
+  if (!r) {
+    delsock(m->fd);
+  }
+  return(1);
 }
 
 char *gnick(char *in)
@@ -1718,6 +1726,7 @@ O_IRC_FUNC(inap)
     ns = sk->fd;
   
   ircmode = 0;
+  lpbrk = 0;   /* clear previous loop interrupt request */
   r = parseout(ns, cstr(str, 1), win);
   ircmode = t;
 }
