@@ -388,8 +388,16 @@ mhdr_t *gethdr(char *fn, int md5blocks)
     } 
     spf = (m.id) ? 1152 : 576;
     r->len = numframes * spf / r->freq;
+
     /* avg bitrate in kbps = (number of kilobits)/(number of seconds) */
+    if (r->len == 0) {  /* patch #442669, jpavel */
+      /* We have a problem here. */
+      close(f);
+      free(r);
+      return(NULL);
+    } 
     r->bitrate = (r->sz1/r->len)/125;
+
   } else {
     /* assume bitrate is constant */
     r->bitrate = bratetab[m.id][m.brate];
