@@ -48,11 +48,10 @@
 #endif
 
 /* standard napster servers */
-char *std_serverlist = "server.napster.com;64.124.41.16;64.124.41.17;64.124.41.18;64.124.41.19";
 
 extern int ircin;
 extern int ircsock;
-extern int ups[];
+extern int ups[], speeds[];
 extern scrls_t *mscroll, *scur;
 extern chans_t *recent;
 extern WINDOW *wchan, *winput;
@@ -234,10 +233,12 @@ char *strip(char *s) {
 char *getline(FILE *f) {
   char *buf;
   char *res;
-  int size;        /* current allocated size of buf */
+  int size;     /* current allocated size of buf */
   int l;
+  size = 200;   /* should be large enough to hold an average line from
+		 * the shared library without having to realloc() the
+		 * buffer */
 
-  size = 100;
   buf = (char *)malloc(size);
 
   res = fgets(buf, size, f);
@@ -270,6 +271,8 @@ int readcfg(char *fn) {
   int other_user = 0;
   char *user=NULL, *pass=NULL, *email=NULL;
   int r;
+  struct stat st;
+  char *gfn;
 
   if (getval("user")) { /* user was specified on command line - do not
                            use password or email from config file */
@@ -286,19 +289,47 @@ int readcfg(char *fn) {
   /* read the config file - but in a wimpy way that does not overwrite
      existing values. This is because command line values should have
      priority. */
-  wp(NULL, "Reading config file %s...\n", fn);
+  wp(NULL, "Reading user config file %s...\n", fn);
   r = loadsets(fn, NULL, 0, 0);
 
   if (r==0) {
     /* success */
   } else if (r==1) {
-    wp(NULL, "There were some warnings, please edit your config file.\n", fn);
+    wp(NULL, "There were some warnings, please edit your config file.\n");
   } else if (errno == ENOENT) {   /* no such file or directory */
-    wp(NULL, "Config file not found.\n", fn);
+    wp(NULL, "%s: file not found.\n", fn);
   } else {
     wp(NULL, "Error loading config file %s: %s\n", fn, strerror(errno));
   }
 
+  /* read the global config file, if any. Note that this is read
+     *last*. I.e., the values given there are the *least*
+     authoritative. */
+
+  gfn = getval("globalconfigfile");
+  if (!gfn) {
+    gfn = GLOBALCONFIGFILE;
+  }
+
+  if (gfn) {
+    r = stat(gfn, &st);
+    if (r==0 && S_ISREG(st.st_mode)) { /* only if file exists and is regular */
+      
+      wp(NULL, "Reading global config file %s...\n", gfn);
+      r = loadsets(gfn, NULL, 0, 0);
+      
+      if (r==0) {
+	/* success */
+      } else if (r==1) {
+	wp(NULL, "There were some warnings, please edit global config file.\n");
+      } else if (errno == ENOENT) {   /* no such file or directory */
+	wp(NULL, "%s: file not found.\n", gfn);
+      } else {
+	wp(NULL, "Error loading global config file %s: %s\n", gfn, strerror(errno));
+      }
+    }
+  }
+
   /* restore password and email if necessary */
   if (other_user) {
     chset("user", user);
@@ -462,8 +493,25 @@ int set_defaults_interactive(char *fn) {
   ans = NULL;
 
   if (!getval("dataport")) {
-    chset("dataport", "6699-6799");
-    wp(NULL, "No dataport given - using %s\n", getval("dataport"));
+    if (!info.daemon) {
+      wp(NULL, "Are you behind a firewall? [y=yes, n=no, u=unsure]: ");
+      ans = getline(stdin);
+      if (ans && ans[0]=='y') {
+	wp(NULL, "Firewalled client - setting dataport to 0\n");
+	chset("dataport", "0");
+      } else if (ans && ans[0]=='n') {
+	wp(NULL, "Using dataport 6699-6799\n");
+	chset("dataport", "6699-6799");
+      } else {
+	wp(NULL, "Okay, I will assume that you are not firewalled. If you\n"
+	   "find out later that you are, please set your dataport to 0.\n");
+	wp(NULL, "Using dataport 6699-6799\n");
+	chset("dataport", "6699-6799");
+      }
+    } else {
+      chset("dataport", "6699-6799");
+      wp(NULL, "No dataport given - using %s\n", getval("dataport"));
+    }
   }
 
   if (!getval("connection")) {
@@ -607,6 +655,7 @@ static struct option longopts[] = {
   {"notitle",     0, 0, 't'},
   {"reconnect",   0, 0, 'r'},
   {"nxterm",      0, 0, 'l'},
+  {"transparent", 0, 0, 'T'},
   {"create",      0, 0, 'm'},
   {"config",      1, 0, 'f'},
   {"server",      1, 0, 's'},
@@ -643,6 +692,7 @@ void phelp(char *nm)
   printf("-t, --notitle      - do not display the title bar (fixes messed-up displays)\n");
   printf("-l, --nxterm       - try using a terminal which is compatible with most\n"
 	 "                     systems (fixes some messed-up displays)\n");
+  printf("-T, --transparent  - use the terminal's default background instead of black\n");
   printf("-f fn, --config fn - specifies the config file to use (default $HOME/"CONFIGFILE")\n");
   printf("-x fn, --log fn    - log all transfers to a specific filename\n");
   printf("-g fn, --logall fn - log everything to a specific filename\n");
@@ -671,28 +721,31 @@ void dopts(int argc, char **argv)
   char *p;
   
   /* defaults */
+  info.name = argv[0];
   info.user = NULL;
   info.pass = NULL;
   info.email = NULL;
   info.up = NULL;
   info.down = NULL;
   info.dataport = NULL;
-  info.conn = NULL;
-  info.d = "0";
-  info.shared_filename = home_file(LIBRARYFILE);
+  info.shared_filename = NULL;
   info.logallfile = NULL;
   info.logfile = NULL;
   info.daemon = 0;
   info.nxterm = 0;
   info.autorestart = 0;
   info.serverlist = NULL;
-  info.maxuploads = NULL;
   info.build = 0;
   info.create = 0;
   info.reconnect = 0;
   info.notop = 0;
+  info.transparent = 0;
+  info.bandwidthdown = -1;
+  info.bandwidthup = -1;
+  info.bandwidthdownconn = -1;
+  info.bandwidthupconn = -1;
 
-  while ((c = getopt_long(argc, argv, "h?vbBNmrqatlf:x:g:s:d:u:p:e:U:D:P:C:M:o:", 
+  while ((c = getopt_long(argc, argv, "h?vbBNmrqatlTf:x:g:s:d:u:p:e:U:D:P:C:M:o:", 
 			  longopts, NULL)) != -1)
   {
     switch (c)
@@ -749,6 +802,9 @@ void dopts(int argc, char **argv)
     case 'l':
       info.nxterm = 1;
       break;
+    case 'T':
+      info.transparent = 1;
+      break;
     case 'g':
       info.logallfile = strdup(optarg);
       break;
@@ -797,10 +853,10 @@ void dopts(int argc, char **argv)
       info.dataport = strdup(optarg);
       break;
     case 'C':
-      info.conn = strdup(optarg);
+      chset("connection", optarg);
       break;
     case 'M':
-      info.maxuploads = strdup(optarg);
+      chset("maxuploads", optarg);
       break;
     default:
       wp(NULL, "Invalid option -- %c\n", c);
@@ -982,6 +1038,39 @@ char *next_server(char *serverlist) {
   }
 }
 
+/* return the "connection speed", a number between 0-10. For the most
+   part, we trust the number given by the user; however, if the user
+   has put a serious bandwidth limit on uploads, then we do not allow
+   her to claim she has a high connection speed. Note: this is not
+   perfect, since the connection speed is only announced to the server
+   at connect time, and the bandwidth limit might be changed
+   later. But what can we do? */
+
+int connection(void) {
+  int c;
+  int limit=0;
+
+  c = nvar("connection");
+  if (c<0 || c>10) {
+    c = 0;
+  }
+  
+  if (info.bandwidthup != 0) {
+    limit = info.bandwidthup;
+  }
+  if (info.bandwidthupconn != 0) {
+    if (!limit || info.bandwidthupconn < limit)
+      limit = info.bandwidthupconn;
+  }
+  if (limit <= 0)
+    return c;
+
+  while (limit < speeds[c]) 
+    c--;
+
+  return c;
+}
+
 /* the main procedure: read command line, parse configuration file,
    check and create various things and then enter event loop; afterwards 
    shut down and quit */
@@ -994,6 +1083,7 @@ int main(int argc, char *argv[])
   FILE *fl;
   char *fn;
   const char *errmsg;
+  char *libraryfile;
 
   /* initialize main screen's internal state */
   cbuf[0] = 0;
@@ -1032,22 +1122,24 @@ int main(int argc, char *argv[])
 
   free(fn);
 
-  if (info.serverlist == NULL) {
-    info.serverlist = strdup(std_serverlist);
+  if (info.shared_filename) {
+    libraryfile = info.shared_filename;
+  } else {
+    libraryfile = home_file(LIBRARYFILE);
   }
 
   if (info.build != -1) {
     if (info.build)
       {
 	wp(NULL, "Building library...\n");
-	if (buildflist(info.shared_filename, info.up) == -1) {
+	if (buildflist(libraryfile, info.up) == -1) {
 	  wp(NULL, "There was an error building your library.\n");
 	  return(1);
 	}
-      } else if (!up_to_date(info.shared_filename, info.up))
+      } else if (!up_to_date(libraryfile, info.up))
 	{
 	  wp(NULL, "Your library is not up-to-date. Rebuilding...\n");
-	  if (buildflist(info.shared_filename, info.up) == -1) {
+	  if (buildflist(libraryfile, info.up) == -1) {
 	    wp(NULL, "There was an error building your library.\n");
 	    return(1);
 	  }
@@ -1057,11 +1149,11 @@ int main(int argc, char *argv[])
       return(1);
   }
 
-  fl = fopen(info.shared_filename, "r");
+  fl = fopen(libraryfile, "r");
   if (!fl)
   {
     wp(NULL, "Error reading %s: %s\nTry running \"nap -b\".\n", \
-       info.shared_filename, strerror(errno));
+       libraryfile, strerror(errno));
     return(1);
   }
   fclose(fl);
@@ -1165,9 +1257,7 @@ int main(int argc, char *argv[])
       continue;
     }
   
-    c = info.conn ? atoi(info.conn) : 0;
-    if (c < 0 || c > 10)
-      c = 0;
+    c = connection();
 
     if (info.create)
     {
@@ -1333,8 +1423,8 @@ int isshared(char *fn, char *sd) {
 int buildflist(char *sd, char *path)
 {
   FILE *f;
-  char sv[513], *t, *n;       /* ### static limit */
-  char *r;
+  char *t, *n;
+  dev_inode_t *di_list=NULL, *e;
 
   f = fopen(sd, "w");
   if (f == NULL)
@@ -1346,14 +1436,6 @@ int buildflist(char *sd, char *path)
   fprintf(f, NAP_LIBRARY_HEADER "\n");
   fprintf(f, "PATH=%s\n", path ? path : "");
 
-  r = getcwd(sv, 512);
-  if (r == NULL)
-  {
-    wp(NULL, "Error getting the current working directory: %s\n", 
-       strerror(errno));
-    return(-1);
-  }
-  
   /* note: if path==NULL, we just generate an empty file */
 
   if (path) {
@@ -1365,7 +1447,10 @@ int buildflist(char *sd, char *path)
       {
 	t = strip(t);  /* remove whitespace on both ends */
 	t = home_file(t);
-	dir(t, f);
+	if (t[0] && t[strlen(t)-1]=='/') {  /* strip '/' from end */
+	  t[strlen(t)-1]=0;
+	}
+	addfile(t, f, &di_list);
 	free(t);
 	t = strtok(NULL, ";");
       }
@@ -1374,67 +1459,74 @@ int buildflist(char *sd, char *path)
   }
 
   fclose(f);
-  
-  chdir(sv);
+
+  list_forall_unlink(e, di_list) {
+    free(e);
+  }
   
   return(1);
 }
 
-/* add the files from directory nm to the library file f */
-void dir(char *nm, FILE *f)
-{
-  DIR *t;
-  char *p=NULL;
-  
-  if (nm[strlen(nm)-1] != '/')
-    msprintf(&p, "%s/", nm);
-  else
-    p = strdup(nm);
-  
-  t = opendir(p);
-  if (!t)
-    return;
-  
-  ddir(p, t, f);
-  closedir(t);
-  free(p);
-}
+/* add the file or directory nm to the library file f. Be careful
+   to avoid loops due to symlinks: DI_LIST is a linked list of
+   device/inode pairs (of directories) that we have already seen. */
 
-/* add the files from directory p (whose pathname is bpath, ending in
-   "/"), to the library file f */
-void ddir(char *bpath, DIR *p, FILE *f)
+void addfile(char *fn, FILE *f, dev_inode_t **di_list_p)
 {
-  struct dirent *ls;
-  DIR *t;
-  char *nbpath=NULL;
+  DIR *dir;
+  struct stat st;
+  int ret;
+  dev_inode_t *e;
   mhdr_t *m;
-  
-  while ((ls = readdir(p)))
-  {
-    msprintf(&nbpath, "%s%s", bpath, ls->d_name);
-    t = opendir(nbpath);  /* a bad way for testing for a directory */
-    if (t)
-    {
-      closedir(t);
-      if (strcmp(ls->d_name, "..") && strcmp(ls->d_name, "."))
-      {
-        free(nbpath);
-        nbpath=NULL;
-        msprintf(&nbpath, "%s%s/", bpath, ls->d_name);
-        dir(nbpath, f);
+  struct dirent *dirent;
+
+  ret = stat(fn, &st);
+  if (ret) { /* ignore files that don't exist */
+    return;
+  }
+
+  if (S_ISDIR(st.st_mode)) {  /* is it a directory? */
+
+    /* do not recurse if we've seen this dir before */
+    list_find(e, *di_list_p, e->dev == st.st_dev && e->ino == st.st_ino);
+    if (e) {
+      return;
+    }
+
+    /* else, remember dir */
+    e = (dev_inode_t *)malloc(sizeof(dev_inode_t));
+    e->dev = st.st_dev;
+    e->ino = st.st_ino;
+    list_prepend(*di_list_p, e);
+
+    /* now go through each file in this directory. */
+    dir = opendir(fn); /* open directory */
+    if (dir==NULL) {   /* ignore failures */
+      return;
+    }
+    while ((dirent = readdir(dir)) != NULL) { /* go through each entry */
+      /* make sure to ignore ".." and "." */
+      if (strcmp(dirent->d_name, "..")!=0 && strcmp(dirent->d_name, ".")!=0) {
+	int len = strlen(fn)+strlen(dirent->d_name)+2;
+	char filename[len]; /* Note: this goes on the stack */
+	strcpy(filename, fn);
+	strcat(filename, "/");
+	strcat(filename, dirent->d_name);
+	addfile(filename, f, di_list_p);
       }
     }
-    else
-    {
-      m = gethdr(nbpath, 293);
-      if (m)
-      {
-        fprintf(f, "\"%s\" %s-%i %i %i %i %i\n", nbpath, m->check, m->sz1, m->sz, m->bitrate, m->freq, m->len);
-        free(m);
-      }
+    closedir(dir);
+    return;
+
+  } else if (S_ISREG(st.st_mode)) { /* is it a regular file? */
+
+    m = gethdr(fn, 293);
+    if (m) {
+      fprintf(f, "\"%s\" %s-%i %i %i %i %i\n", fn, m->check, m->sz1, m->sz, m->bitrate, m->freq, m->len);
+      free(m);
     }
-    free(nbpath);
-    nbpath=NULL;
+    return;
+
   }
 }
 
@@ -1713,7 +1805,7 @@ int connect_t(int fd, struct sockaddr *serv_addr, int addlen, int t)
      up if the child terminates */
   signal(SIGCHLD, noop);
   if (t>0)
-    sleep(t);
+    t = sleep(t);
   signal(SIGCHLD, SIG_IGN);
 
   r = waitpid(pid, &status, WNOHANG);
@@ -1723,7 +1815,12 @@ int connect_t(int fd, struct sockaddr *serv_addr, int addlen, int t)
     return(-1);
   } else if (r==0) {
     kill(pid, SIGTERM);
-    errno = ETIMEDOUT;
+    waitpid(pid, &status, 0);      /* wait for the child to exit */
+    if (t>0) {
+      errno = EINTR;
+    } else {
+      errno = ETIMEDOUT;
+    }
     return(-1);
   } else if (WEXITSTATUS(status)) {
     errno = WEXITSTATUS(status);
@@ -1840,7 +1937,8 @@ int conn(char *host, unsigned short port)
   }
   
   wp(wchan, "Connecting...\n");
-  
+  drw(wchan);
+
   if (connect_t(s, (struct sockaddr *)&dst, sizeof(dst), timeout) == -1)
   {
     wp(wchan, "Error connecting to %s:%d: %s\n", buf, port, errno!=255 ? strerror(errno) : "unknown error");
@@ -1859,9 +1957,16 @@ int login(int s, char *user, char *pass, int data, char *client, int conn, char
   char *t1=NULL;
   phead_t *rcv;
   int r;
+  char *libraryfile;
 
   msprintf(&t1, "nap v%s", client);
-  sendpack(s, F_LOGIN, "%s %s %i \"%s\" %i", user, pass, data, t1, conn);
+  r = sendpack(s, F_LOGIN, "%s %s %i \"%s\" %i", user, pass, data, t1, conn);
+  if (r<0) {
+    wp(wchan, "Error while sending data: %s\n", strerror(errno));
+    drw(wchan);
+    return(-1);
+  }
+
   free(t1);
   t1 = NULL;
   r = recvpack(s, &t1, &rcv);
@@ -1869,7 +1974,7 @@ int login(int s, char *user, char *pass, int data, char *client, int conn, char
     r = recvpack(s, &t1, &rcv);
 
   if (r == -1) {
-    wp(wchan, "Error: %s\n", strerror(errno));
+    wp(wchan, "Error while receiving data: %s\n", strerror(errno));
     drw(wchan);
     return(-1);
   } else if (r == -3) {
@@ -1894,7 +1999,13 @@ int login(int s, char *user, char *pass, int data, char *client, int conn, char
   free(t1);
   free(rcv);
   
-  return(lfiles(s, info.shared_filename));
+  if (info.shared_filename) {
+    libraryfile = info.shared_filename;
+  } else {
+    libraryfile = home_file(LIBRARYFILE);
+  }
+
+  return(lfiles(s, libraryfile));
 }
 
 int makeact(int s, char *user, char *pass, int data, char *client, int conn, char *email)
@@ -1902,7 +2013,8 @@ int makeact(int s, char *user, char *pass, int data, char *client, int conn, cha
   char *t1=NULL;
   phead_t *rcv;
   int r;
-  
+  char *libraryfile;
+
   sendpack(s, F_MKUSER, user);
   r = recvpack(s, &t1, &rcv);
   while (r == -2)
@@ -1981,7 +2093,13 @@ int makeact(int s, char *user, char *pass, int data, char *client, int conn, cha
   free(t1);
   free(rcv);
   
-  return(lfiles(s, info.shared_filename));
+  if (info.shared_filename) {
+    libraryfile = info.shared_filename;
+  } else {
+    libraryfile = home_file(LIBRARYFILE);
+  }
+
+  return(lfiles(s, libraryfile));
 }
 
 /* calculate the name of the hotlist file */
@@ -2253,7 +2371,7 @@ int ssock(int s, const char *fmt, ...)
 
 /* receive a packet from the server (at file descriptor s). Return -1
    if there was an error (with errno set), or -3 if there was an
-   unexpected end-of-file). Return -2 if only part of the packet could
+   unexpected end-of-file. Return -2 if only part of the packet could
    be read; in this case, a subsequent call to recvpack will continue
    to read the same packet. Return 1 on success */
 int recvpack(int s, char **buf, phead_t **hdr)
@@ -2294,6 +2412,13 @@ int recvpack(int s, char **buf, phead_t **hdr)
     *hdr = thdr;
     *buf = strdup("");
     thdr = NULL;
+
+    /* print messages with length 0 */
+    if (nvar("debug") == 2) {
+      wp(wchan, ""DARK GREEN"<-- (0x%x=%d) <>"WHITE"\n", (*hdr)->op, (*hdr)->op);
+      drw(wchan);
+    }
+
     return(1);
   }
   
