@@ -28,6 +28,7 @@ typedef struct phead_s phead_t;
 
 struct info_s 
 {
+  char *name;            /* argv[0], the filename by which nap was invoked */
   char *user;            /* our Napster login name */
   char *pass;            /* our Napster password */
   char *email;           /* our Napster email address */
@@ -37,9 +38,6 @@ struct info_s
   char *serverlist;      /* ';'-separated list of servers to connect to */
   char *logallfile;      /* name of general log file (or NULL) */
   char *logfile;         /* name of transfer log file (or NULL) */
-  char *maxuploads;      /* max number of simultaneous uploads - an integer */
-  char *conn;            /* connection speed 0-9 */
-  char *d;               /* debug level - an integer */
   char *dataport;        /* port or range of ports to use for client-to-client
 			    connections */
   int autorestart;       /* auto reconnect when connection to server lost? */
@@ -56,6 +54,11 @@ struct info_s
   int create;            /* create new account? This is reset to 0 after acct created */
   int reconnect;         /* keep reconnecting until connection established? */
   int notop;             /* if set, display no title */
+  int bandwidthdown;     /* bandwidth limit for total downloads */
+  int bandwidthup;       /* bandwidth limit for total uploads */
+  int bandwidthdownconn; /* bandwidth limit for individual downloads */
+  int bandwidthupconn;   /* bandwidth limit for individual uploads */
+  int transparent;       /* let the terminal background shine through? */
 };
 typedef struct info_s info_t;
 
@@ -89,13 +92,13 @@ void dopts(int, char **);
 int setup_configdir(void);
 int move_file(const char *oldpath, const char *newpath);
 char *next_server(char *serverlist);
+int connection(void);
 int main(int argc, char *argv[]);
 
 int rebuild(int, char *, char *);
 int isshared(char *fn, char *sd);
 int buildflist(char *, char *);
-void dir(char *, FILE *);
-void ddir(char *, DIR *, FILE *);
+void addfile(char *fn, FILE *f, dev_inode_t **di_list_p);
 int lfiles(int, char *);
 int up_to_date(char *sd, char *path);
 int modified_after(time_t mtime, char *fn, dev_inode_t **di_list_p);
