@@ -14,6 +14,7 @@
 #include <unistd.h>
 #include <sys/socket.h>
 #include <errno.h>
+#include <sys/time.h>
 
 #include "defines.h"
 #include "codes.h"
@@ -35,6 +36,9 @@ extern WINDOW *wchan;
 /* the global connection list */
 static sock_t *socklist = NULL;
 
+/* bandwidth observers for global uploads and global downloads */
+bandwidth_t bwup, bwdown;
+
 /* set by /quit to cause an immediate unconditional shutdown of nap,
    or by qevent() after a /tquit */
 int quit_now = 0;
@@ -58,6 +62,11 @@ sock_t *addsock(int s, char *nm, unsigned char t, int (*func)(WINDOW *, sock_t *
   cur->func = func;
   cur->next = NULL;
   cur->dxx = NULL;
+  cur->utask = NULL;
+  cur->dtask = NULL;
+  cur->btask = NULL;
+
+  cur->bwlimit = 0;
 
   /* an add it to the list */
   list_append(sock_t, socklist, cur);
@@ -110,14 +119,9 @@ void delsock(int s)
   free(cur->nm);
   free(cur);
 
-  /* In many cases, the socket is closed by a connection handler before
-   * the call to delsock(). (See the many places in event.c where a
-   * connection handler calls close(m->fd) before return(0)).
-   * It would be safer to only call close() once, but this second call 
-   * to close() hasn't caused problems so far because delsock() is always
-   * called immediately after a connection handler returns 0. i.e. there
-   * is no opportunity for the socket descriptor to be reused in the mean
-   * time (or is there?) */
+  /* Note: a connection handler may not close the socket before or
+   * after the call to delsock(), since it is always closed here, and
+   * it can be harmful (segfault) to close a file twice. */
   if (s > 2) {
     shutdown(s, 2);
     close(s);
@@ -156,10 +160,83 @@ void psocks(WINDOW *win)
   drw(win);
 }
 
-/* this is the main event loop of the nap program. It is here that we
- * wait for network activity and then tend to the various different
- * connections (including user input). In addition, we call tevent()
- * (in timer.c) once a second or so, to take care of scheduled events.
+/* the next few functions are for bandwidth limiting. The idea is that
+   a bandwidth "observer" is created for each bandwidth limit. Every
+   time bytes travel through the connection, the observer is
+   informed. The observer calculates, on demand, the number of
+   microseconds one has to wait until the next packet may be
+   sent/received. */
+
+/* register n bytes that have been sent and are subject to bw */
+void bandwidth_register(bandwidth_t *bw, int n) {
+  bw->bytes += n;
+}
+
+/* initialize bw */
+void bandwidth_init(bandwidth_t *bw) {
+  bw->bandwidth = 0;
+}
+
+/* return the time in microseconds that this connection has to sleep
+   in order to satisfy its bandwidth constraint. curbw is the
+   bandwidth currently requested by the user; note that this may be
+   different from bw->bandwidth if the requested bandwidth changed, or
+   the very first time this is called for a connection. tp is a
+   pointer to a timeval structure that holds the current time. */
+
+long bandwidth_waittime(bandwidth_t *bw, int curbw, struct timeval *tp) {
+  long ms;
+  
+  if (curbw < 0)
+    curbw = 0;
+
+  /* First check if the requested bandwidth differs from the stored
+     one. If yes, re-calculate state. */
+  
+  if (bw->bandwidth != curbw) {
+    if (bw->bandwidth == 0) {
+      bw->bandwidth = curbw;
+      bw->time0 = tp->tv_sec;
+      bw->bytes = tp->tv_usec / 1000 * curbw;
+    } else {
+      bw->bytes = (bw->bytes / bw->bandwidth) * curbw;
+      bw->bandwidth = curbw;
+    }
+  }
+
+  if (curbw==0)
+    return 0;       /* no bandwidth limit requested */
+
+  /* next, adjust the time0 parameter to tp->tv_sec, i.e., to the
+     integer component of the current time. */
+  if (tp->tv_sec != bw->time0) {
+    bw->bytes -= curbw * 1000 * (tp->tv_sec - bw->time0);
+    bw->time0 = tp->tv_sec;
+  }
+
+  /* calculate the number of microseconds this connection needs to
+     sleep */
+  ms = bw->bytes / curbw * 1000 - tp->tv_usec;
+
+  /* if we are more than a second behind, forfeit anything beyond one
+     second of credit. I.e., normally we get credit for being "too
+     slow" (and can make up for it later), but at most one second's
+     worth of such credit is allowed */
+  
+  if (ms < -1000000) {
+    bw->bytes = (tp->tv_usec / 1000 - 1000) * curbw;
+    ms = bw->bytes / curbw * 1000 - tp->tv_usec;
+  }
+  
+  return ms;
+}
+
+
+/* sockfunc: this is the main event loop of the nap program. It is
+ * here that we wait for network activity and then tend to the various
+ * different connections (including user input). In addition, we call
+ * tevent() (in timer.c) once a second or so, to take care of
+ * scheduled events.
  *
  * This function is called precisely once, from nap.c:main(). Once
  * we leave here, nap shuts down and terminates.
@@ -169,9 +246,9 @@ void psocks(WINDOW *win)
  * global variable wchan). Why not just refer to the global variables,
  * as do some of the procedures that are called from here?
  *
- * Here are the current conventions for what the socket readers
- * (cur->func) are supposed to return. They normally return 1. A
- * return value of 0 means the current socket should be deleted.
+ * The connection handlers (cur->func) are always supposed to return 1
+ * (the return value is not currently checked). If a connection
+ * handler wants to shut down, it needs to call delsock().
  * 
  **/
 
@@ -183,7 +260,12 @@ void sockfunc(WINDOW *win, WINDOW *up)
   struct timeval sec; /* note originally used <time.h> timespec here
 			 instead of <select.h> timeval - this was a
 			 type error */
-  
+  long timeout, msup, msdown;
+  struct timeval tv;
+
+  bandwidth_init(&bwup);
+  bandwidth_init(&bwdown);
+
   while (1)
   {
     dochecks();  /* notify terminal that it has been resized, if
@@ -212,6 +294,15 @@ void sockfunc(WINDOW *win, WINDOW *up)
     if (quit_now)
       return;
 
+    /* determine current time */
+    gettimeofday(&tv, NULL);
+
+    msup = bandwidth_waittime(&bwup, info.bandwidthup, &tv);
+    msdown = bandwidth_waittime(&bwdown, info.bandwidthdown, &tv);
+
+    timeout = 1000000;  /* let select() time out after at most this many
+                           microsecs */
+
     /* let fs be the set of "read" file descriptors, and fw the list
        of "write" file descriptors, from the connection list */
 
@@ -219,17 +310,48 @@ void sockfunc(WINDOW *win, WINDOW *up)
     FD_ZERO(&fw);
     for (cur=socklist;cur!=NULL;cur=cur->next)
     {
+      if (cur->bwlimit) {
+	/* bandwidth limiting code */
+	int ms;
+
+	/* calculate the sleep time for this connection */
+	if (cur->t == S_R) {
+	  ms = bandwidth_waittime(&cur->bw, info.bandwidthdownconn, &tv);
+	  if (msdown > ms) {
+	    ms = msdown;
+	  }
+	} else {
+	  ms = bandwidth_waittime(&cur->bw, info.bandwidthupconn, &tv);
+	  if (msup > ms) {
+	    ms = msup;
+	  }
+	}
+	
+	/* if ms<=0, activity is immediately allowed and we proceed to
+	   watch this connection in the "select" call below. Else if
+	   ms>0, activity is not allowed, but we make sure "select"
+	   will time out after ms microseconds so that this connection
+	   can be added at that time. */
+
+	if (ms>0) {
+	  if (timeout > ms) {
+	    timeout = ms;
+	  }
+	  continue;
+	}
+      }
+
       if (cur->t == S_R)
         FD_SET(cur->fd, &fs);
       else if (cur->t == S_W)
         FD_SET(cur->fd, &fw);
     }
     
-    /* prepare timeout of 1 second, then wait for activity on connections.
-       On fs connections, wait for characters to be available for reading.
+    /* prepare timeout, then wait for activity on connections.  On fs
+       connections, wait for characters to be available for reading.
        On fw connections, wait for ability to write. */
-    sec.tv_sec = 1;
-    sec.tv_usec = 0;
+    sec.tv_sec = timeout / 1000000;
+    sec.tv_usec = timeout % 1000000;
     r = select(FD_SETSIZE, &fs, &fw, NULL, &sec);
 
     /* timeout guarantees calling the following at least once per second */
@@ -259,26 +381,20 @@ void sockfunc(WINDOW *win, WINDOW *up)
       if (cur->t == S_R && FD_ISSET(cur->fd, &fs))
       {
 	FD_CLR(cur->fd, &fs);
-        r = cur->func(win, cur);
+        cur->func(win, cur);
 	if (quit_now) {
 	  goto quit;
 	}
-        if (!r) {
-          delsock(cur->fd);
-	}
 	cur=socklist;  /* need to start from beginning since sockets might
 		      have been deleted in the meantime */
       }
       else if (cur->t == S_W && FD_ISSET(cur->fd, &fw))
       {
 	FD_CLR(cur->fd, &fw);
-        r = cur->func(win, cur);
+        cur->func(win, cur);
 	if (quit_now) {
 	  goto quit;
 	}
-        if (!r) {
-          delsock(cur->fd);
-        }
 	cur=socklist;
       }
     }
@@ -286,7 +402,7 @@ void sockfunc(WINDOW *win, WINDOW *up)
 
   /* end of loop not reachable */
 
- quit:                 
+ quit:
   /* we get here if the user issues the /quit command. */
   while (socklist!=NULL)
     delsock(socklist->fd);
