@@ -64,8 +64,10 @@ extern int quit_now, quit_after_transfers, upsocks, downsocks;
 
 extern ssearch_t *search;
 extern int srch;
+extern int noping;
 extern upload_t *up;
 extern download_t *down;
+extern struct inbrowse_s directbrowse;
 extern int wmode;
 extern unsigned char *dwi;
 extern int fl;
@@ -73,10 +75,17 @@ extern int fl;
 /* names of the different connection types, 0-10 */
 const char *conns[] = { "Unknown", "14.4", "28.8", "33.6", "56k", "ISDN-64K",
                   "ISDN-128K", "Cable", "DSL", "T1", "T3" };
+
 /* how many simultaneous uploads we allow by default, for each
    connection type */
 int ups[] = { 3, 2, 3, 3, 4, 5, 5, 7, 7, 9, 9 };
 
+/* what kind of minimal download speed we expect, roughly, from each
+   connection type. Note: this is in kilobytes/second, not
+   kilobits/second. The only place this is used is in
+   nap.c:connection(). */
+int speeds[] = { 0, 1, 1, 2, 3, 4, 8, 8, 8, 125, 2500 };
+
 int tin = 0;
 scount_t scount;           /* libraries / songs / gigs */
 
@@ -92,6 +101,9 @@ in_nap_cmd_t in[] = {
 	 { F_FREQ, sfreq },
 	 { F_SSF, ssf },
 	 { F_COUNT, sscount },
+         { F_BROWSE2ACC, sbrowse2acc },
+         { F_BROWSE2ERR, sbrowse2err },
+         { F_BROWSE2, sbrowse2req },
 	 { F_SWHOIS, swhois },
 	 { F_SOFF, soff },
 	 { F_UON, suon },
@@ -180,30 +192,73 @@ tvsub(register struct timeval *out, register struct timeval *in)
 	out->tv_sec -= in->tv_sec;
 }
 
-/* receive ping response */
+/* receive ping response from "napping" process */
 int icmpin(WINDOW *win, sock_t *m)
 {
-  int s = m->fd;
   ssearch_t *cur;
-  char buf[256];
-  struct sockaddr_in frm;
-  int frmlen = sizeof(frm);
-  struct timeval tv;
+  char *line, *p;
+  struct in_addr addr;
+  long ms;
+  int r;
+
+  line = getline(m->f);
+
+  if (!line) {
+    fclose(m->f);
+    delsock(m->fd);
+    showresults(wchan, 2);
+    return(1);
+  }
+
+  if (strncmp(line, "IP", 2) == 0) {
+    wp(win, "* Waiting for ping results...\n");
+    drw(win);
+    for (cur=search; cur!=NULL; cur=cur->next) {
+      cur->ping = -1;
+    }
+
+    return(1);
+  }
+
+  p = strchr(line, ' ');
+  if (!p) {  /* syntax error from napping, ignore */
+    return(1);
+  }
+  *p = 0;
+  p++;
+  r = inet_aton(line, &addr);
+  if (!r) {  /* syntax error from napping, ignore */
+    return(1);
+  }
+  ms = strtol(p, NULL, 10);
   
-  gettimeofday(&tv, NULL);
-  
-  memset(buf, 0, sizeof(buf));
-  recvfrom(s, buf, sizeof(buf), 0, (struct sockaddr *)&frm, &frmlen);
-  
-  for (cur=search; cur!=NULL; cur=cur->next)
-  {
-    if (cur->nip == frm.sin_addr.s_addr && !timerisset(&cur->r))
-      memcpy(&cur->r, &tv, sizeof(tv));
+  for (cur=search; cur!=NULL; cur=cur->next) {
+    if (cur->nip == addr.s_addr && cur->ping == -1) {
+      cur->ping = ms/1000;
+    }
   }
   
   return(1);
 }
 
+/* receive error message from "napping" process */
+int icmperr(WINDOW *win, sock_t *m)
+{
+  char *line;
+
+  line = getline(m->f);
+
+  if (line) {
+    wp(win, "* Could not obtain ping results: %s\n", line);
+    drw(win);
+  }
+  
+  fclose(m->f);
+  delsock(m->fd);
+
+  return(1);
+}
+
 char **form_toks(char *buf, int *cnt)
 {
   int i, j, k, l = 0, c=0;
@@ -888,7 +943,7 @@ I_NAP_FUNC(snend)
    all servers send the <speed>.  */
 I_NAP_FUNC(sfreq)
 {
-  char *lfn, *fn;
+  char *lfn, *fn, *libraryfile;
   upload_t *task;
   int i, r;
 
@@ -905,7 +960,13 @@ I_NAP_FUNC(sfreq)
      This was a major security hole, since it would allow a remote
      client to download any file, whether it was shared or not */
 
-  if (!isshared(lfn, info.shared_filename)) {
+  if (info.shared_filename) {
+    libraryfile = info.shared_filename;
+  } else {
+    libraryfile = home_file(LIBRARYFILE);
+  }
+
+  if (!isshared(lfn, libraryfile)) {
     sendpack(s, F_GFR, "%s", str);  /* see comment below */
     wp(win, "%s* File \"%s\" was requested by %s, but is not shared!%s\n", RED, lfn, tok[0], WHITE);
     drw(win);
@@ -1189,7 +1250,7 @@ I_NAP_FUNC(ssret)
   /* create a new search list element */
   elt = (ssearch_t *)malloc(sizeof(ssearch_t));
   
-  elt->song = strdup(t);
+  elt->song = strdup(quote(t));
   elt->fn = strdup(tok[0]);
   elt->sz = atoi(tok[2]);
   elt->brate = atoi(tok[3]);
@@ -1200,9 +1261,7 @@ I_NAP_FUNC(ssret)
   elt->conn = atoi(tok[8]);
   if (elt->conn > 10 || elt->conn < 0)
     elt->conn = 0;
-  timerclear(&elt->s);
-  timerclear(&elt->r);
-  elt->ping = -2;
+  elt->ping = -3;
   elt->next = NULL;
   
   /* and add it to the list. Note that it is faster to order the list
@@ -1217,85 +1276,137 @@ I_NAP_FUNC(ssret)
 I_NAP_FUNC(ssend)
 {
   ssearch_t *cur, *cur1;
-  char *desc;
-  char *d;
   sock_t *sk;
-  struct icmphdr *icmp;
-  struct sockaddr_in dst;
-  
-  sk = findsock("icmp");  /* icmp is the socket for ping results, 
-			     if they were requested */
-  
-  if (!sk) {
-    showresults(win, 1);
-    return(1);
-  }
+  int pin[2], pout[2], perr[2];
+  FILE *fin;
+  struct in_addr addr;
+  char *args[2];
+  char *napping;
+  int i;
 
-  if (!search) {
-    delsock(sk->fd);
-    
-    showresults(win, 1);
-    return(1);
-  }
-
-  /* send out pings */
-  for (cur=search; cur; cur=cur->next) {
-    if (timerisset(&cur->s))
-      continue;
-    
-    dst.sin_addr.s_addr = cur->nip; 
-    dst.sin_family = AF_INET;
-    
-    d = (char *)malloc(64);
-    
-    /* the following code will not work on big-endian machines. */
-    memset(d, 0, 64);
-    icmp = (struct icmphdr *)d;  
-    icmp->type = ICMP_ECHO;
-    icmp->code = 0;
-    icmp->un.echo.id = (getpid()&0xffff);
-    icmp->un.echo.sequence = 0;
-    icmp->checksum = in_cksum((u_short *)icmp, 64);
-    
-    sendto(sk->fd, d, 64, 0, (struct sockaddr *)&dst, sizeof(dst));
-    gettimeofday(&cur->s, NULL);
-    free(d);
-    
-    for (cur1=search;cur1;cur1=cur1->next)
-      if (cur1->nip == cur->nip)
-	memcpy(&cur1->s, &cur->s, sizeof(cur->s));
-  }
-  
-  wp(win, "* Waiting for ping results...\n");
+  list_length(ssearch_t, search, i);
+  wp(win, "* Received %d search %s\n", i, i==1 ? "result" : "results");
   drw(win);
+
+  /* the easy case is if we can skip sending out pings. */
+  if (noping || !search) {
+    showresults(win, 1);
+    return(1);
+  }
+
+  /* attempt to run "napping" application, which will collect ping
+     information for us. */
   
-  /* set timer for 3 seconds */
-  desc = strdup("[waiting for ping results]");
-  addtimer(3, timed_pingresults, (void *)NULL, desc);
-  return (1);
+  /* first create pipes to and from */
+  /* do we need to do anything to avoid file descriptors 0-2? */
+  if (pipe(pin) < 0 || pipe(pout) < 0 || pipe(perr) < 0) {
+    wp(win, "* Could not collect ping results. pipe: %s\n", strerror(errno));
+    drw(win);
+    showresults(win, 1);
+    return(1);
+  }
+
+  switch(fork()) {
+  case -1: /* error */
+    wp(win, "* Could not collect ping results. fork: %s\n", strerror(errno));
+    drw(win);
+    showresults(win, 1);
+    return(1);
+    break;
+
+  case 0: /* child */
+    /* attach pipes to stdin, stdout, stderr */
+    close(pin[1]);
+    close(pout[0]);
+    close(perr[0]);
+    dup2(pin[0], 0);
+    dup2(pout[1], 1);
+    dup2(perr[1], 2);
+    close(pin[0]);
+    close(pout[1]);
+    close(perr[1]);
+
+    napping = getval("napping");
+    if (!napping)
+      napping = NAPPING;
+
+    args[0] = napping;
+    args[1] = NULL;
+
+    execvp(napping, args);
+
+    /* execlp only returns if there is an error. */
+    if (errno == ENOENT && getval("napping")==NULL) {
+      /* if napping was neither found nor explicitly requested, just
+	 ignore it silently and proceed without pings */
+      exit(1);
+    } else if (errno == EACCES) {
+      fprintf(stderr, "%s is not executable\n", napping);
+      exit(1);
+    } else {
+      fprintf(stderr, "%s: %s\n", napping, strerror(errno));
+      exit(1);
+    }
+    break;
+
+  default: /* parent */
+    close(pin[0]);
+    close(pout[1]);
+    close(perr[1]);
+
+    fin = fdopen(pin[1], "w");
+    
+    /* send the IP addresses to napping */
+    for (cur=search; cur; cur=cur->next) {
+      if (cur->ping == -2)
+	continue;
+      
+      addr.s_addr = cur->nip; 
+      fprintf(fin, "%s\n", inet_ntoa(addr));
+            
+      for (cur1=search;cur1;cur1=cur1->next)
+	if (cur1->nip == cur->nip)
+	  cur1->ping = -2;
+    }
+    fprintf(fin, "\n");
+    fclose(fin);
+    
+    /* hook up sockets to receive ping results */
+    addsock(pout[0], "icmp", S_R, icmpin);
+    addsock(perr[0], "icmperr", S_R, icmperr);
+    sk = findsock("icmp");
+    sk->f = fdopen(dup(pout[0]), "r");
+    sk = findsock("icmperr");
+    sk->f = fdopen(dup(perr[0]), "r");
+
+    /* set timer for 3 seconds */
+    /* timer is not currently used, since napping, if installed, times
+       out after 3 seconds anyway. */
+    /* 
+       desc = strdup("[waiting for ping responses]");
+       addtimer(3, timed_pingresults, (void *)NULL, desc);
+    */
+    return (1);
+  }
 }
 
+/* timer is not currently used, since napping, if installed, times
+   out after 3 seconds anyway. */
 void timed_pingresults(void *dummy) {
   sock_t *sk;
-  ssearch_t *cur;
-  struct timeval tv;
 
   sk = findsock("icmp");
-  if (sk)
+  if (sk) {
+    fclose(sk->f);
     delsock(sk->fd);
-  
-  /* compute ping times (cur->r - cur->s) */
-  for (cur=search;cur;cur=cur->next) {
-    if (timerisset(&cur->r)) {
-      memcpy(&tv, &cur->r, sizeof(tv));
-      tvsub(&tv, &cur->s);
-      cur->ping = (tv.tv_sec*10000+(tv.tv_usec/100))/10;
-      timerclear(&tv);
-    }
-    else
-      cur->ping = -1;
   }
-  
+
+  sk = findsock("icmperr");
+  if (sk) {
+    fclose(sk->f);
+    delsock(sk->fd);
+  }
+
   showresults(wchan, 2);
 }
 
@@ -1332,8 +1443,6 @@ void showresults(WINDOW *win, int level) {
   if (nvar("noresultscreen")==1) 
   {
     if (!search) {
-      wp(win, "* Received 0 %s results\n", level==0 ? "browse" : "search");
-      drw(win);
       return;
     }
 
@@ -1362,13 +1471,10 @@ void showresults(WINDOW *win, int level) {
   }
   else 
   {
-    list_length(ssearch_t, search, i);
-    wp(win, i==1 ? "* Received %d %s result\n" : "* Received %d %s results\n",
-       i, level==0 ? "browse" : "search");
-    drw(win);
-    if (i>0)
+    if (search) {
       resetsscr();
       switchtoscreen(RESULT_SCREEN);
+    }
     plist();
   }
 } 
@@ -1418,10 +1524,139 @@ I_NAP_FUNC(srbrowse)
  * <nick> [ip] */
 I_NAP_FUNC(sdbrowse)
 {
+  int i;
+
+  list_length(ssearch_t, search, i);
+  wp(win, "* Received %d browse %s\n", i, i==1 ? "result" : "results");
+  drw(win);
+
   showresults(win, 0);
   return(1);
 }
 
+/* 642 (0x282) direct browse error: <nick> "message" */
+I_NAP_FUNC(sbrowse2err)
+{
+  if (num < 2)
+    return(1);
+
+  srch = 0;
+  directbrowse.state = FAILED;
+
+  wp(win, "* Unable to directly browse %s: %s\n", tok[0], tok[1]);
+  drw(win);
+  return(1);
+}
+
+/* 641 (0x281) direct browse accept: <nick> <ip> <port>
+ * <nick> accepted our direct browse request and the server is giving us 
+ * the IP address and port to connect to. 
+ * Note: If <nick> is firewalled, we will not receive this packet.
+ * The remote client will connect to us and issue a SENDLIST command
+ * (assuming we are not also firewalled). */
+I_NAP_FUNC(sbrowse2acc)
+{
+  struct sockaddr_in dst;
+  int k;
+  sock_t *sk=NULL;
+  unsigned short int dstport;
+  
+  if (num < 3) /* error, ignore the message */
+    return(1);
+
+  if (directbrowse.state != REQUESTED || \
+      strcasecmp(directbrowse.nick, tok[0]) != 0) {
+    /* didn't ask to browse this nick (or maybe we did, but that request
+     * has timed out). Just ignore the message. */
+    return(1);
+  }
+  
+  dstport = htons(atoi(tok[2]));
+  if (dstport == 0) {
+    return(1); 
+  }
+  
+  /* connect to the remote client */
+  dst.sin_addr.s_addr = htonl(swapl(strtoul(tok[1], (char **)NULL, 10)));
+  dst.sin_port = dstport;
+  dst.sin_family = AF_INET;
+
+  k = socket(AF_INET, SOCK_STREAM, 0);
+  if (fork()==0) {     /* child process */
+    connect(k, (struct sockaddr *)&dst, sizeof(dst));
+    exit(1);
+  }
+  /* parent process */
+  sk = addsock(k, "dirbrowse", S_R, initgetlist); /* initgetlist in event.c */
+  
+  directbrowse.state = CONNECTING;
+  directbrowse.sk = sk;
+
+  if (nvar("debug") == 2) {
+    wp(win, ""DARK GREEN"* Connecting to %s to get list of shared files"\
+        WHITE"\n", tok[0]);
+    drw(win);
+  }
+
+  return(1);
+}
+
+/* 640 (0x280) direct browse request: <nick> [ip port] 
+ * <nick> is requesting to directly browse our shared files.
+ * If we are firewalled, the server also provides <nick>'s IP address and port
+ * so that we can initiate the connection. */
+I_NAP_FUNC(sbrowse2req)
+{
+  struct sockaddr_in dst;
+  int k;
+  sock_t *sk=NULL;
+  unsigned short int dstport;
+  
+  if (num < 1)
+    return(1);
+
+  /* XXX if we limit the number of outgoing browse connections,
+   * check that we are within the limit here. */
+
+  if (num < 3) {
+    /* accept the direct browse request: <nick> */
+    sendpack(s, F_BROWSE2ACC, "%s", tok[0]); 
+    /* remote client will initiate the connection */
+    return(1); 
+  }
+
+  /* Note the Napster v2.0 BETA 8 & BETA 9 clients don't send a
+   * 641 direct browse accept message if they are firewalled. They
+   * simply connect to the remote client and issue a SENDLIST command.
+   * (They also don't understand SENDLIST commands that are issued
+   * to them, but that is a separate matter...) */
+
+  dstport = htons(atoi(tok[2]));
+  if (dstport == 0)
+    return(1); /* this is an error but we ignore it */
+
+  /* initiate connection because we are firewalled */
+  dst.sin_addr.s_addr = htonl(swapl(strtoul(tok[1], (char **)NULL, 10)));
+  dst.sin_port = dstport;
+  dst.sin_family = AF_INET;
+
+  k = socket(AF_INET, SOCK_STREAM, 0);
+  if (fork()==0) {     /* child process */
+    connect(k, (struct sockaddr *)&dst, sizeof(dst));
+    exit(1);
+  }
+  /* parent process */
+  sk = addsock(k, gnum(2), S_R, initsendlist); /* initsendlist in event.c */
+
+  if (nvar("debug") == 2) {
+    wp(win, ""DARK GREEN"* Connecting to %s to send list of shared files"\
+        WHITE"\n", tok[0]);
+    drw(win);
+  }
+  
+  return(1);
+}
+
 I_NAP_FUNC(sop)
 {
   wp(win, "%s!%s%s%s!%s %s\n", GREEN, WHITE, tok[0], GREEN, WHITE, str+strlen(tok[0])+1);
@@ -1521,9 +1756,7 @@ I_NAP_FUNC(sredir)
   free(t1);
   wp(win, "Logging in...\n");
   drw(win);
-  c = info.conn ? atoi(info.conn) : 0;
-  if (c < 0 || c > 10)
-    c = 0;
+  c = connection();
   if (login(t, info.user, info.pass, info.port, CLIENT, c, info.email) == -1)
   {
     drw(win);
@@ -1571,9 +1804,7 @@ I_NAP_FUNC(scycle)
   }
   wp(win, "Logging in...\n");
   drw(win);
-  c = info.conn ? atoi(info.conn) : 0;
-  if (c < 0 || c > 10)
-    c = 0;
+  c = connection();
   if (login(t, info.user, info.pass, info.port, CLIENT, c, info.email) == -1)
   {
     drw(win);
