@@ -92,7 +92,7 @@ void resetsscr(void)
      information is not available */
 
   showconn = (showconn && search && search->conn != -1) ? 1 : 0;
-  showping = (showping && search && search->ping != -2) ? 1 : 0;
+  showping = (showping && search && search->ping > -2) ? 1 : 0;
 
   /* reset screen and cursor to first item (after possible sorting!) */
   cy = 1; 
@@ -274,7 +274,7 @@ void plist()
       {
 	if (status)
 	  free(status);
-	status = strdup(cur->fn);
+	status = strdup(quote(cur->fn));
       }
     }
     
@@ -511,6 +511,11 @@ int shandlechar(chtype cbuf)
     }
     break;
 
+  case 12:                              /* Ctrl-L */
+    clearok(swin, 1);
+    wrefresh(swin);
+    break;
+
   case '\n':
     if (search)
     {
