@@ -26,6 +26,7 @@
 #include "alias.h"
 #include "irc.h"
 #include "lists.h"
+#include "scheck.h"
 
 extern info_t info;
 extern int ircmode, wmode;
@@ -34,7 +35,7 @@ extern upload_t *up;    /* download task list */
 extern download_t *down;    /* download task list */
 extern scount_t scount;     /* libraries, songs, gigs */
 extern char *mnick;
-extern int noprint;
+extern int noprint, lpbrk;
 extern int ircsock;
 extern alias_t *alhead;     /* alias list */
 extern out_nap_cmd_t out[]; /* list of builting commands */
@@ -273,10 +274,12 @@ void dstatus()
   }
 
 #ifdef MCURSES
-  if (cloaked)
-    mvwprintw(sep, 0, 0, "\e[0;37;44m (%s) [%i songs in %i libraries (%i gigs)] [nap v%s] [U/%i D/%i Q/%i]", info.user, scount.songs, scount.libraries, scount.gigs, VERSION, nu, nd, ndq);
-  else
-    mvwprintw(sep, 0, 0, "\e[0;37;44m [%s] [%i songs in %i libraries (%i gigs)] [nap v%s] [U/%i D/%i Q/%i]", info.user, scount.songs, scount.libraries, scount.gigs, VERSION, nu, nd, ndq);
+  mvwprintw(sep, 0, 0, "\e[0;37;44m ");
+  wprintw(sep, cloaked ? "(%s)" : "[%s]", info.user);
+  wprintw(sep, scount.songs == 1 ? " [%lu song" : " [%lu songs", scount.songs);
+  wprintw(sep, scount.libraries == 1 ? " in %lu library" : " in %lu libraries", scount.libraries);
+  wprintw(sep, scount.gigs == 1 ? " (%lu gig)]" : " (%lu gigs)]", scount.gigs);
+  wprintw(sep, " [nap v%s] [U/%i D/%i Q/%i]", VERSION, nu, nd, ndq);
   mvwaddstr(sep, 0, COLS+10, "\e[0m");
   drw(sep);
 #else
@@ -295,10 +298,10 @@ void dstatus()
     msprintf(&t, "[U/%i D/%i Q/%i]", nu, nd, ndq);
     mvwprintw(sep, 0, COLS-(strlen(t)+1), t);
     free(t);
-    if (cloaked)
-      mvwprintw(sep, 0, 0, " (%s) [%lu songs in %lu libraries (%lu gigs)]", info.user, scount.songs, scount.libraries, scount.gigs);
-    else
-      mvwprintw(sep, 0, 0, " [%s] [%lu songs in %lu libraries (%lu gigs)]", info.user, scount.songs, scount.libraries, scount.gigs);
+    mvwprintw(sep, 0, 0, cloaked ? " (%s)" : " [%s]", info.user);
+    wprintw(sep, scount.songs == 1 ? " [%lu song" : " [%lu songs", scount.songs);
+    wprintw(sep, scount.libraries == 1 ? " in %lu library" : " in %lu libraries", scount.libraries);
+    wprintw(sep, scount.gigs == 1 ? " (%lu gig)]" : " (%lu gigs)]", scount.gigs);
   }
   else
   {
@@ -425,7 +428,7 @@ void addlog(char *logfile, char *str, int rp)
 
   for (i=0;str[i];i++) {
     if (bol) {
-      fprintf(f, "[%02i/%02i %i:%02i] ", r->tm_mon, r->tm_mday, r->tm_hour, r->tm_min);
+      fprintf(f, "[%02i/%02i %i:%02i] ", r->tm_mon+1, r->tm_mday, r->tm_hour, r->tm_min);
       bol = 0;
     }
     if (str[i] == '\e') {
@@ -888,6 +891,7 @@ int input(WINDOW *win, sock_t *m)
       wp(win, "\n");
       drw(win);
     } else if (*command == '/') {
+      lpbrk = 0;   /* clear previous loop interrupt request */
       n = parseout(s, command+1, win);
       indraw();  /* necessary because command could have been "/query" etc */
       dstatus();
@@ -959,6 +963,12 @@ int input(WINDOW *win, sock_t *m)
     cbuf[curx] = 0;
     curx--;
 
+    while (curx >= 0 && cbuf[curx] == ' ') 
+    { 
+      cbuf[curx] = 0; 
+      curx--; 
+    } 
+
     while (curx >= 0 && cbuf[curx] != ' ')
     {
       cbuf[curx] = 0;
@@ -1211,7 +1221,8 @@ void initwin(unsigned char f)
 {
   struct termios ts;
   WINDOW *w;
-  
+  int bgcolor;
+
   if (info.daemon) {
     info.daemon = 2;
     return;
@@ -1252,14 +1263,23 @@ void initwin(unsigned char f)
   cbreak();
   noecho();
   
+  bgcolor = COLOR_BLACK;
+
+#ifdef HAVE_ASSUME_DEFAULT_COLORS
+  if (info.transparent) {
+    assume_default_colors(COLOR_WHITE, -1);
+    bgcolor = -1;
+  }
+#endif
+
   init_pair(1, COLOR_WHITE, COLOR_BLUE);
-  init_pair(CPR, COLOR_RED, COLOR_BLACK);
-  init_pair(CPG, COLOR_GREEN, COLOR_BLACK);
-  init_pair(CPW, COLOR_WHITE, COLOR_BLACK);
-  init_pair(CPB, COLOR_BLUE, COLOR_BLACK);
-  init_pair(CPY, COLOR_YELLOW, COLOR_BLACK);
-  init_pair(CPM, COLOR_MAGENTA, COLOR_BLACK);
-  init_pair(CPC, COLOR_CYAN, COLOR_BLACK);
+  init_pair(CPR, COLOR_RED, bgcolor);
+  init_pair(CPG, COLOR_GREEN, bgcolor);
+  init_pair(CPW, COLOR_WHITE, bgcolor);
+  init_pair(CPB, COLOR_BLUE, bgcolor);
+  init_pair(CPY, COLOR_YELLOW, bgcolor);
+  init_pair(CPM, COLOR_MAGENTA, bgcolor);
+  init_pair(CPC, COLOR_CYAN, bgcolor);
   init_pair(CPBR, COLOR_WHITE, COLOR_RED);
   winput = newwin("input", 1, 0, LINES-1, 0);
 #else
